/*
 *  Dynamic array of type.
 */

#include <string.h>
#include <stdlib.h>
#include "typearray.h"
#include "util.h"

enum {
	BLOCK_SIZE = 8,
};

TypeArray *new_typearray(void)
{
	TypeArray *array;

	array = zalloc(sizeof(struct TypeArray));
	array->size = 0;
	array->nalloc = 0;
	array->data = 0;
	return array;
}

void del_typearray(TypeArray *array)
{
	zfree(array->data);
	zfree(array);
}

int typearray_append(TypeArray *array, type value)
{
	long nalloc;
	type *data;

	if (array->nalloc == 0) {
		array->data = zalloc(BLOCK_SIZE * sizeof(value));
		if (! array->data)
			return -1;
		array->nalloc = BLOCK_SIZE;
	}
	else if (array->size >= array->nalloc) {
		nalloc = array->nalloc + BLOCK_SIZE;
		data = zrealloc(array->data, nalloc * sizeof(value));
		if (! data)
			return -1;
		array->data = data;
		array->nalloc += BLOCK_SIZE;
	}
	array->data[array->size++] = value;
	return array->size;
}

int typearray_remove(TypeArray *array, long which)
{
	type *data;

	data = array->data;

	if (which >= array->size)
		return -1;

	if (array->size - which - 1 > 0)
		memmove(&data[which], &data[which+1], array->size - which-1);
	array->size--;
	return array->size;
}

