#!/usr/local/bin/python

# Generate the function prototypes and operand-mappings for
# the Assembler, from data in the valid.txt file.

import os, sys, string

def main():
	generate('valid.txt', 'valid.c')

def generate(srcfile, validfile):
	data = string.split(open(srcfile).read())
	gen_valid(validfile, data)

def gen_valid(validfile, data):
	table = {}
	operation = []
	for word in data:
		if is_number(word):
			opcode = word
			operation = [opcode]
			table[opcode] = operation
		elif len(operation) == 1:
			opname = word
			operation.append(opname)
		else:
			opmodes = word
			opfunc, optypes = modes_to_func(opname, opmodes)
			operation.append([opmodes, opfunc, optypes])
	print_valid(validfile, table)

def print_valid(validfile, table):
	header = """/*
 *  Assembler
 *
 *  Valid Operand Modes Table
 *  Do not edit this machine generated file (use genvalid.py)
 */

#include "asm.h"
"""
	f = open(validfile, 'w')
	f.write(header)
	f.write('\n')

	opcodes = table.keys()
	opcodes.sort()
	for opcode in opcodes:
		t = table[opcode]
		f.write('static ValidMode ' + t[1] + '_valid [] = {\n')
		for tuple in t[2:]:
			f.write("\t{ ")
			m, count = mode_to_valid(tuple[0])
			f.write('"' + m + '"')
			#f.write(", (Func) " + tuple[1])
			f.write(", " + str(count))
			f.write(" }")
			f.write(",")
			f.write("\n")
		f.write('\t{ "",   -1 }\n')
		f.write('};\n\n')

	f.write('ValidMode * opvalid [256] = {\n')
	for i in range(256):
		opcode = "0x%02.2X" % i
		if opcode in opcodes:
			t = table[opcode]
			f.write('\t' + t[1] + '_valid')
		else:
			f.write('\tNULL')
		if i != 255:
			f.write(',')
		if (i+1) % 4 == 0:
			f.write('\n')
		elif opcode not in opcodes:
			f.write('\t')
	f.write('};\n')

def mode_to_valid(mode):
	m = ""
	count = 0
	for ch in mode:
		if ch == 'p':
			ch = '-'
		elif ch == '+':
			ch = '-'
		elif ch == '-':
			ch = '-'
		else:
			count = count + 1
		m = m + ch
	return (m, count)


def modes_to_func(opname, opmodes):
	func = ''
	types = []
	if len(opmodes) != 3:
		print "an opmode wasn't of length 3, hmmm"
		exit(-1)
	for i in [0, 1]:
		ch = opmodes[i]
		if ch == '+':
			ch = opmodes[i+1]
		func = func + operand_char(ch)
		types.append(operand_type(ch))
	for i in [2]:
		ch = opmodes[i]
		if ch == '+':
			ch = opmodes[i+1]
		func = func + dst_operand_char(ch)
		types.append(dst_operand_type(ch))
	if func != '':
		func = opname + '_' + func
	else:
		func = opname
	return func, types

def operand_char(ch):
	if ch == 'A':
		ch = 'abs'
	elif ch == 'S':
		ch = 'rel'
	elif ch == 'R':
		ch = 'abs'
	#elif ch == 'p':
	#	ch = ''
	elif ch == '-':
		ch = ''
	elif ch == 'L':
		ch = 'i'
	elif ch == 'l':
		ch = 'i'
	elif ch == 'r':
		ch = 'i'
	elif ch == 'c':
		ch = 'c'
	elif ch == 'i':
		ch = 'i'
	elif ch == 'd':
		ch = 'd'
	elif ch == 's':
		ch = 's'
	elif ch == 'a':
		ch = 'a'
	else:
		print "Hmm, char in opmode was wrong (ch=" + ch + ")"
		sys.exit(-1)
	return ch

def dst_operand_char(ch):
	#if ch == 'A':
	#	ch = 'abs'
	#elif ch == 'S':
	#	ch = 'rel'
	#elif ch == 'R':
	#	ch = 'abs'
	#elif ch == 'p':
	#	ch = ''
	if ch == '-':
		ch = ''
	#elif ch == 'L':
	#	ch = 'i'
	elif ch == 'l':
		ch = ''
	elif ch == 'r':
		ch = 'r'	# use register * for dst
	elif ch == 'c':
		ch = 'c'
	elif ch == 'i':
		ch = 'i'
	elif ch == 'd':
		ch = 'd'
	elif ch == 's':
		ch = 's'
	elif ch == 'a':
		ch = 'a'
	else:
		print "Hmm, dst char in opmode was wrong (ch=" + ch + ")"
		sys.exit(-1)
	return ch

def operand_type(ch):
	if ch == 'A':
		ch = 'Int *'
	elif ch == 'S':
		ch = 'Int *'
	elif ch == 'R':
		ch = 'Int *'
	#elif ch == 'p':
	#	ch = 'Proc *'
	elif ch == '-':
		ch = 'void *'
	elif ch == 'L':
		ch = 'Int *'
	elif ch == 'l':
		ch = 'Int *'
	elif ch == 'r':
		ch = 'Int *'
	elif ch == 'c':
		ch = 'Cell *'
	elif ch == 'i':
		ch = 'Int *'
	elif ch == 'd':
		ch = 'Double *'
	elif ch == 's':
		ch = 'String *'
	elif ch == 'a':
		ch = 'Array *'
	else:
		print "Hmm, char in opmode was wrong (ch=" + ch + ")"
		sys.exit(-1)
	return ch

def dst_operand_type(ch):
	#if ch == 'A':
	#	ch = 'Int *'
	#elif ch == 'S':
	#	ch = 'Int *'
	#elif ch == 'R':
	#	ch = 'Int *'
	#elif ch == 'p':
	#	ch = 'Proc *'
	if ch == '-':
		ch = 'void *'
	elif ch == 'l':
		ch = 'void *'	# can't have literal as dst except using L mode
	elif ch == 'r':
		ch = 'Int *'
	elif ch == 'c':
		ch = 'Cell *'
	elif ch == 'i':
		ch = 'Cell *'
	elif ch == 'd':
		ch = 'Cell *'
	elif ch == 's':
		ch = 'Cell *'
	elif ch == 'a':
		ch = 'Array *'
	else:
		print "Hmm, dst char in opmode was wrong (ch=" + ch + ")"
		sys.exit(-1)
	return ch

def is_number(word):
	if len(word) < 2:
		return 0
	if word[:2] != '0x':
		return 0
	for ch in word[2:]:
		if ch not in '0123456789abcdefABCDEF':
			return 0
	return 1

def hex_to_int(word):
	value = 0
	if len(word) < 2:
		return -1
	if word[:2] != '0x':
		return -2
	word = string.lower(word)
	hexdigits = '0123456789abcdef'
	for ch in word[2:]:
		if ch not in hexdigits:
			return -3
		value = value * 16
		value = value + string.find(hexdigits, ch)
	return value

if __name__ == '__main__':
	main()
