/*
 *  Assembler Main function:
 */

#include "asm.h"

/*
 *  Load the program:
 */
int main(int argc, char **argv)
{
	FILE *in, *out;
	int i, len;
	int debugging = 0;
	char *infile = NULL;
	char *outfile = NULL;
	char *name, *s;
	Program p;

	for (i=1; i < argc; i++) {

		if (strcmp(argv[i], "debug=on") == 0) {
			debugging = 1;
			continue;
		}
		else if (strcmp(argv[i], "debug=off") == 0) {
			debugging = 0;
			continue;
		}

		infile = argv[i];

		for (name=s=infile; *s; s++) {
			/* find file's base name */
			if ((*s == '/') || (*s == '\\'))
				name = s+1;
		}
		len = strlen(name);
		if (len < 5) {
			error("input filename \"%s\" was too short", infile);
			continue;
		}

		if ((strcmp(&name[len-4], ".asm") != 0) &&
		    (strcmp(&name[len-4], ".ASM") != 0))
		{
			error("input filename \"%s\" must end in .asm", infile);
			continue;
		}

		in = fopen(infile, "r");
		if (in == NULL) {
			error("could not open input file \"%s\"", infile);
			continue;
		}

		if (outfile)
			zfree(outfile);

		outfile = zalloc(len+1);
		strcpy(outfile, name);
		strcpy(&outfile[len-4], ".avm");

		if (debugging)
			note("assembling %s to %s\n", infile, outfile);

		p = parse_program(in);
		fclose(in);

		if (debugging)
			print_program(p);

		if (p->error_count > 0)
			error("parsing errors.");

		if (! gen_program(p, outfile)) {
			error("assembler errors, skipping this file...");
			continue;
		}

		out = fopen(outfile, "w");
		if (out != NULL) {
			output_program(p, out);
			fclose(out);
		}
		else
			error("could not open output file \"%s\"", outfile);
	}

	if (argc == 1) {
		note("Ace Assembler\n");
		note("usage: asm [debug=on] file.asm [file2.asm ...]\n");
		note("output: creates an avm if there are no errors in the source program.\n");
	}

	return 0;
}

