/*
 *  Parser definitions:
 */

  enum {
	MAX_ERRORS = 10,	// maximum errors before skipping file
  };

  typedef struct Expr * Expr;
  typedef struct Operand Operand;
  typedef struct Code * Code;
  typedef struct Program * Program;

  struct Expr {	 		// an expression
	int	kind;		// IDENT, INTVAL, REG, CELL
	Token	token;		// leaf value (label, int, reg, icell etc)
	int	op;		// PLUS, MINUS, LSHIFT, BITWISE-NOT etc
	Expr	left;		// left-hand sub-expression (if any)
	Expr	right;		// right-hand sub-expression
  };

  struct Operand {		// an instruction operand
	Token	token;		// 56, r15, [r4]
	Expr	expr;		// or a constant expression
  };

  struct Code {			// an instruction
	Token	op;		// instruction opcode name and value
	short	noperands;	// number of operands in list
	Operand	operands[3];	// list of operands (0 up to 3)
	long	hexcode;	// encoded representation
  };

  struct Program {		// an assembly language program
	FILE *		file;
	long		addr;	// latest instruction or memory cell address
	int		error_count;
	CodeArray *	codes;		// an array of Code
	ExprArray *	exprs;		// an array of Expr (int expressions)
	IntArray *	ints;		// an array of long
	DoubleArray *	doubles;	// an array of double
	StringArray *	strings;	// an array of char *
	long		Ncode, Nint, Ndouble, Nstring;
  };

/*
 *  Parser functions:
 */

Program	parse_program(FILE *infile);
Expr	parse_expr(Program p);

