/*
 *  Dynamic array of (void *). This code can store any
 *  kind of object as long is you have a pointer to the
 *  the object.
 */

#include <string.h>
#include <stdlib.h>
#include "ptrarray.h"
#include "util.h"

enum {
	BLOCK_SIZE = 8,
};

PtrArray *new_ptrarray(void)
{
	PtrArray *array;

	array = zalloc(sizeof(struct PtrArray));
	array->size = 0;
	array->nalloc = 0;
	array->data = 0;
	return array;
}

void del_ptrarray(PtrArray *array)
{
	zfree(array->data);
	zfree(array);
}

long ptrarray_append(PtrArray *array, void * value)
{
	long nalloc;
	void **data;

	if (array->nalloc == 0) {
		array->data = zalloc(BLOCK_SIZE * sizeof(value));
		if (! array->data)
			return -1;
		array->nalloc = BLOCK_SIZE;
	}
	else if (array->size >= array->nalloc) {
		nalloc = array->nalloc + BLOCK_SIZE;
		data = zrealloc(array->data, nalloc * sizeof(value));
		if (! data)
			return -1;
		array->data = data;
		array->nalloc += BLOCK_SIZE;
	}
	array->data[array->size++] = value;
	return array->size;
}

long ptrarray_remove(PtrArray *array, long which)
{
	void **data;

	data = array->data;

	if (which >= array->size)
		return -1;

	if (array->size - which - 1 > 0)
		memmove(&data[which], &data[which+1], array->size - which-1);
	array->size--;
	return array->size;
}

