/*
 *  Dynamic array of strings (actually char *).
 */

#include <string.h>
#include <stdlib.h>
#include "strarray.h"
#include "util.h"

enum {
	BLOCK_SIZE = 8,
};

StringArray *new_stringarray(void)
{
	StringArray *array;

	array = zalloc(sizeof(struct StringArray));
	array->size = 0;
	array->nalloc = 0;
	array->data = 0;
	return array;
}

void del_stringarray(StringArray *array)
{
	zfree(array->data);
	zfree(array);
}

long stringarray_append(StringArray *array, char * value)
{
	long nalloc;
	char **data;

	if (array->nalloc == 0) {
		array->data = zalloc(BLOCK_SIZE * sizeof(value));
		if (! array->data)
			return -1;
		array->nalloc = BLOCK_SIZE;
	}
	else if (array->size >= array->nalloc) {
		nalloc = array->nalloc + BLOCK_SIZE;
		data = zrealloc(array->data, nalloc * sizeof(value));
		if (! data)
			return -1;
		array->data = data;
		array->nalloc += BLOCK_SIZE;
	}
	array->data[array->size++] = value;
	return array->size;
}

long stringarray_remove(StringArray *array, long which)
{
	char **data;

	data = array->data;

	if (which >= array->size)
		return -1;

	if (array->size - which - 1 > 0)
		memmove(&data[which], &data[which+1], array->size - which-1);
	array->size--;
	return array->size;
}

