/*
 *  Memory management.
 */

#include "asm.h"

/* zalloc and zfree */

void * zalloc (long size)
{
	void *ptr;

	ptr = malloc(size);
	if (ptr == NULL) {
		error("zalloc: ran out of memory");
		exit(-1);
	}
	memset(ptr, 0, size);
	return ptr;
}

void * zrealloc(void *ptr, long size)
{
	ptr = realloc(ptr, size);
	if (ptr == NULL) {
		error("zrealloc: ran out of memory");
		exit(-1);
	}
	return ptr;
}

char * zstrdup(char *cs)
{
	char *s;

	s = zalloc(sizeof(char) * (strlen(cs)+1));
	strcpy(s, cs);
	return s;
}

void zfree (void *ptr)
{
	free(ptr);
}

/*
 *  A function for checking that a string only contains whitespace
 *  and one integer, returns 0 if the string doesn't contain just
 *  an integer, or 1 if it contains optional whitespace, followed
 *  by one integer, followed by optional whitespace.
 */
int string_is_int(char *s)
{
	int state, ch;

	for (state = 1; *s != '\0'; s++) {
		ch = *s;
		if (state == 1) {
			/* expect whitespace or + or - or digit */
			if (isspace(ch))
				state = 1;
			else if (ch == '+')
				state = 2;
			else if (ch == '-')
				state = 2;
			else if (isdigit(ch))
				state = 3;
			else
				return 0;
		}
		else if (state == 2) {
			/* expect digits now */
			if (isdigit(ch))
				state = 3;
			else
				return 0;
		}
		else if (state == 3) {
			/* expect digits or whitespace or end */
			if (isdigit(ch))
				state = 3;
			else if (isspace(ch))
				state = 4;
			else
				return 0;
		}
		else if (state == 4) {
			/* expect only whitespace or end */
			if (isspace(ch))
				state = 4;
			else
				return 0;
		}
	}
	if (state >= 3)
		return 1;
	else
		return 0;
}

